package permission

import (
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestPermission_RemoveFields(t *testing.T) {

	tests := []struct {
		name            string
		unallowedFields []string
		in              map[string]interface{}
		want            map[string]interface{}
	}{
		{
			name: "nil",
			in:   nil,
			want: nil,
		},
		{
			name: "empty",
			in:   map[string]interface{}{},
			want: map[string]interface{}{},
		},
		{
			name:            "empty unallowedFields",
			in:              map[string]interface{}{"f": "v"},
			want:            map[string]interface{}{"f": "v"},
			unallowedFields: nil,
		},
		{
			name:            "remove fields",
			in:              map[string]interface{}{"f": "v", "f1": "v"},
			want:            map[string]interface{}{"f": "v"},
			unallowedFields: []string{"f1"},
		},
		{
			name:            "all present fields allowed",
			in:              map[string]interface{}{"f": "v", "f1": "v"},
			want:            map[string]interface{}{"f": "v", "f1": "v"},
			unallowedFields: []string{"f2"},
		},
		{
			name:            "unallowed fields in nested object",
			in:              map[string]interface{}{"obj": map[string]interface{}{"f": "v", "f1": "v"}},
			want:            map[string]interface{}{"obj": map[string]interface{}{"f": "v"}},
			unallowedFields: []string{"obj.f1"},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			p := Permission{
				UnallowedFields: tt.unallowedFields,
			}
			assert.Equal(t, tt.want, p.RemoveFields(tt.in))
		})
	}
}
